unit DICE_Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Menus, Vcl.ExtCtrls, Vcl.StdCtrls, NumEdits,
  Vcl.MPlayer, XorShift;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileMnu: TMenuItem;
    FileOpenMnu: TMenuItem;
    FileSaveMnu: TMenuItem;
    FileSaveAsMnu: TMenuItem;
    FileN1Mnu: TMenuItem;
    FileResetMnu: TMenuItem;
    FileN2Mnu: TMenuItem;
    FileExitMnu: TMenuItem;
    ModeMnu: TMenuItem;
    ModeNumLotteryMnu: TMenuItem;
    ModeStrLotteryMnu: TMenuItem;
    ModeStrExtractMnu: TMenuItem;
    ModeN1Mnu: TMenuItem;
    ModeMuteMnu: TMenuItem;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    ModePanel: TPanel;
    NumberPanel: TPanel;
    QuantityEdt: TUpDownEdit;
    RateLbl: TLabel;
    FaceEdt: TUpDownEdit;
    StringPanel: TPanel;
    PluckMeo: TMemo;
    Splitter: TSplitter;
    ResultPanel: TPanel;
    CountBtn: TButton;
    ResultEdt: TEdit;
    MediaPlayer: TMediaPlayer;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure FileOpenMnuClick(Sender: TObject);
    procedure FileSaveMnuClick(Sender: TObject);
    procedure FileSaveAsMnuClick(Sender: TObject);
    procedure FileResetMnuClick(Sender: TObject);
    procedure FileExitMnuClick(Sender: TObject);
    procedure ModeMnusClick(Sender: TObject);
    procedure ModeMuteMnuClick(Sender: TObject);
    procedure ModePanelResize(Sender: TObject);
    procedure NumberEdtsVerify(Sender: TObject; var AText: string; var APosition: Integer;
      Relation: TNumericallyRelationship);
    procedure ResultsCange(Sender: TObject);
    procedure CountBtnClick(Sender: TObject);
  private
    { Private 錾 }
    FGenerator: TXorShift;
    FPluckFile: string;
    function SaveQuery: Integer;
    procedure LoadFile(FileName: string);
    procedure SaveFile(FileName: string);
  protected
    { protected 錾 }
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  System.Types, System.UITypes, FileConsts, FileUtils, System.StrUtils, Winapi.ShellAPI;

function TMainForm.SaveQuery: Integer;
var
  FileName: string;
begin
  if PluckMeo.Modified then
  begin
    FileName := ExtractFileName(FPluckFile);
    if FileName = '' then
      FileName := SUntitled;
    Result := MessageDlg(Format(SSaveChanges, [FileName]), mtWarning, mbYesNoCancel, 0);
  end
  else
    Result := mrNone;
end;

procedure TMainForm.LoadFile(FileName: string);

  procedure LoadList;
  begin
    FPluckFile := FileName;
    PluckMeo.Lines.LoadFromFile(FPluckFile);
    PluckMeo.Modified := False;
  end;

  procedure LoadEffect;
  begin
    MediaPlayer.FileName := FileName;
    MediaPlayer.Open;
  end;

begin
  case SaveQuery of
    mrCancel: Exit;
    mrYes: SaveFile(FPluckFile);
  end;

  if FileName = '' then
  begin
    if OpenDlg.Execute then
      FileName := OpenDlg.FileName
    else
      Exit;
  end;
  if MatchesMasks(FileName, '*.txt') then LoadList
  else
  if MatchesMasks(FileName, '*.wav;*.mp3') then LoadEffect
  else
  begin
    try
      LoadEffect;
    except
      on E: EMCIDeviceError do LoadList;
    end;
  end;
end;

procedure TMainForm.SaveFile(FileName: string);
begin
  if FileName = '' then
  begin
    if SaveDlg.Execute then
      FileName := SaveDlg.FileName
    else
      Exit;
  end;
  FPluckFile := FileName;
  PluckMeo.Lines.SaveToFile(FPluckFile);
  PluckMeo.Modified := False;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  i, j: Integer;
  StrArr: TStringDynArray;
begin
  FGenerator := TXorShift.Create;
  FGenerator.Randomize;
  FPluckFile := '';

  MediaPlayer.Wait := False;
  MediaPlayer.Notify := True;
  ClientHeight := NumberPanel.Height;

  for i := 1 to ParamCount do
  begin
    if CharInSet(ParamStr(i)[1], ['-', '/']) then
    begin
      for j := 2 to High(ParamStr(i)) do
        case ParamStr(i)[j] of
          'n', 'N': ModeNumLotteryMnu.Click;
          's', 'S': ModeStrLotteryMnu.Click;
          'e', 'E': ModeStrExtractMnu.Click;
          'm', 'M': ModeMuteMnu.Checked := True;
        end;
    end
    else if Pos('*', ParamStr(i)) > 0 then
    begin
      StrArr := SplitString(ParamStr(i), '*');
      if Length(StrArr) = 2 then
      begin
        QuantityEdt.Value := StrToIntDef(StrArr[0], 1);
        FaceEdt.Value := StrToIntDef(StrArr[1], 2);
      end;
    end
    else
      LoadFile(ParamStr(i));
  end;
end;

procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  //t@Chbv󂯓B
  Params.ExStyle := Params.EXStyle or WS_EX_ACCEPTFILES;
end;

procedure TMainForm.WMDropFiles(var Msg: TWMDropFiles);
var
  Count, i: Integer;
  FileName: array[0..MAX_PATH] of Char;
begin
  Count := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0);
  try
    for i := 0 to Count - 1 do
    begin
      DragQueryFile(Msg.Drop, i, FileName, SizeOf(FileName));
      LoadFile(FileName);
    end;
  finally
    DragFinish(Msg.Drop);
  end;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  case SaveQuery of
    mrCancel: CanClose := False;
    mrYes: SaveFile(FPluckFile);
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FGenerator.Free;
end;

procedure TMainForm.FileOpenMnuClick(Sender: TObject);
begin
  LoadFile('');
end;

procedure TMainForm.FileSaveMnuClick(Sender: TObject);
begin
  SaveFile(FPluckFile);
end;

procedure TMainForm.FileSaveAsMnuClick(Sender: TObject);
begin
  SaveFile('');
end;

procedure TMainForm.FileResetMnuClick(Sender: TObject);
begin
  MediaPlayer.Close;
end;

procedure TMainForm.FileExitMnuClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ModeMnusClick(Sender: TObject);
begin
  TMenuItem(Sender).Checked := True;

  NumberPanel.Visible := ModeNumLotteryMnu.Checked;
  StringPanel.Visible := not NumberPanel.Visible;
end;

procedure TMainForm.ModeMuteMnuClick(Sender: TObject);
begin
  if ModeMuteMnu.Checked and (MediaPlayer.DeviceID > 0) then
    MediaPlayer.Stop;
end;

procedure TMainForm.ModePanelResize(Sender: TObject);
begin
  // TSplitter.AutoSnap LA⍂ÕRg[łƈʒu֌WȂB
  if ModePanel.Width = 0 then
    Splitter.Left := ModePanel.Left + ModePanel.Width + 1;
end;

procedure TMainForm.NumberEdtsVerify(Sender: TObject; var AText: string; var APosition: Integer;
  Relation: TNumericallyRelationship);
begin
  if Relation <> nrAccord then
  begin
    Beep;
    AText := IntToStr(APosition);
  end;
end;

procedure TMainForm.ResultsCange(Sender: TObject);
begin
  if Canvas.TextWidth(ResultEdt.Text) > (ResultEdt.ClientWidth - 2) then
    ResultEdt.Hint := ResultEdt.Text
  else
    ResultEdt.Hint := '';
end;

procedure TMainForm.CountBtnClick(Sender: TObject);
var
  Total, i: Integer;
begin
  if ModeNumLotteryMnu.Checked then
  begin
    Total := 0;
    for i := 1 to QuantityEdt.Value do
      Inc(Total, FGenerator.Random(FaceEdt.Value) + 1);
    ResultEdt.Text := IntToStr(Total);
  end
  else
  begin
    i := FGenerator.Random(PluckMeo.Lines.Count);
    ResultEdt.Text := PluckMeo.Lines.Strings[i];
    if ModeStrExtractMnu.Checked then
    begin
      PluckMeo.Lines.Delete(i);
      PluckMeo.Modified := False;
    end;
  end;
  if not ModeMuteMnu.Checked then
  begin
    if MediaPlayer.DeviceID > 0 then
    begin
      MediaPlayer.Stop;
      MediaPlayer.Rewind;
      MediaPlayer.Play;
    end
    else
      MessageBeep(MB_ICONASTERISK);
  end;
end;

end.
